<?php
/**
 * Anwiki is a multilingual content management system <http://www.anwiki.com>
 * Copyright (C) 2007-2008 Antoine Walter <http://www.anw.fr>
 * 
 * Anwiki is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 * 
 * Anwiki is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Anwiki.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * ContentFieldsContainer definition.
 * @package Anwiki
 * @version $Id: class_contentclass.php,v 1.22 2008-09-11 22:42:02 antoine-cvs Exp $
 * @copyright 2007-2008 Antoine Walter
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License 3
 */

//TODO : Everything should be static... not possible because of 
//PHP5 bug inherited static attributes shared accross all subclasses...

//TODO : architecture problem : this class should not inherit directly from AnwComponent, this should be ContentClass and ContentClassSettings, 
//but it's not possible without copying a lot of code between classes because multiple inherit is not supported by PHP.
abstract class AnwStructuredContentFieldsContainer extends AnwComponent
{
	private $aoContentFields = array();
	
	//public
	function getContentFields()
	{
		return $this->aoContentFields;
	}
	
	function getContentField($sContentFieldName)
	{
		$aoContentFields = $this->getContentFields();
		if (!isset($aoContentFields[$sContentFieldName]))
		{
			throw new AnwUnexpectedException("ContentField not found : ".$sContentFieldName);
		}
		return $aoContentFields[$sContentFieldName];
	}
	
	//used by action_edit
	/*protected*/ function searchContentField($sContentFieldName)
	{
		try 
		{
			return $this->getContentField($sContentFieldName);
		}
		catch (AnwUnexpectedException $e)
		{
			foreach ($this->getContentFields() as $oContentField)
			{
				if ($oContentField instanceof AnwStructuredContentField_composed)
				{
					try
					{
						return $oContentField->searchContentField($sContentFieldName);
					}
					catch(AnwUnexpectedException $e){}
				}
			}
			throw new AnwUnexpectedException("ContentField not found : ".$sContentFieldName);
		}
	}
	
	//public for plugins
	/*protected*/ function addContentField($oContentField)
	{
		$sFieldName = $oContentField->getName();
		if (isset($this->aoContentFields[$sFieldName]))
		{
			throw new AnwUnexpectedException("ContentField ".$sFieldName." is already defined");
		}
		$oContentField->setContainer($this);
		$oContentField->init();
		$this->aoContentFields[$sFieldName] = $oContentField;
	}
	
	protected static function initContentField($oContentField)
	{
		if ($oContentField instanceof AnwStructuredContentField_composed)
		{
			$oContentField->init();
			$aoSubFields = $oContentField->getContentFields();
			foreach ($aoSubFields as $oSubField)
			{
				//recursive call
				self::initContentField($oSubField);
			}
		}
		else
		{
			$oContentField->init();
		}
	}
	
	abstract function rebuildContentFromXml($sDefaultValue);
	
	//overridable
		
	/*
	function runCallbackOnContentFields($oCallbackInstance, $fCallback, $asParameters, $oContent, $sSuffix, $nInputNumber)
	{
		$oContent = $oCallbackInstance->$fCallback($asParameters, $this, $oContent, $sSuffix, $nInputNumber);
		foreach ($this->getContentFields() as $oSubContentField)
		{
			$oSubContent = $oContent->getSubContent($oSubContentField->getName());
			$sReturn .= $oSubContentField->runCallbackOnContentFields($oCallbackInstance, $fCallback, $asParameters, $oSubContent, $sSuffix, $nInputNumber);
		}
		return $sReturn;
	}*/
	
	//
	
	
}

?>